@@ -9,6 +9,7 @@ import numpy as np
 import math
 import sys
 import six
+import copy
 
 logger = logging.getLogger("expr")
 logger.setLevel(logging.ERROR)
@@ -137,6 +138,9 @@ def pow(left, right):
 def sqr(node):
     return ast.BinOp(left=node, right=num(2), op=ast.Pow())
 
+def sqrt(node):
+    return call('sqrt', [node])
+
 
 def neg(node):
     return ast.UnaryOp(op=ast.USub(), operand=node)
@@ -169,6 +173,11 @@ def _dcos(n, args):
     assert len(args) == 1
     return neg(call('sin', args=args))
 
+def _darccos(n, args):
+    assert n == 0
+    assert len(args) == 1
+    a = sqrt(sub(num(1), sqr(args[0])))
+    return neg(div(num(1), a))
 
 def _darctan2(n, args):
     # derivative of arctan2(y, x)
@@ -187,6 +196,7 @@ standard_function_derivatives['cos'] = _dcos
 standard_function_derivatives['log10'] = _dlog10
 standard_function_derivatives['sqrt'] = _dsqrt
 standard_function_derivatives['arctan2'] = _darctan2
+standard_function_derivatives['arccos'] = _darccos
 
 
 class Derivative(ast.NodeTransformer):
@@ -225,6 +235,10 @@ class Derivative(ast.NodeTransformer):
                 result = add(result, term)
         return result
 
+    def generic_visit(self, node):
+        # it's annoying that the default one modifies in place
+        return super(Derivative, self).generic_visit(copy.deepcopy(node))
+
     def visit_BinOp(self, node):
         solution = None
         if isinstance(node.op, ast.Mult):
@@ -259,6 +273,9 @@ class Derivative(ast.NodeTransformer):
 
 
 class ExpressionString(ast.NodeVisitor):
+    def __init__(self, pretty=False):
+        self.pretty = pretty
+        self.indent = 0
     def visit_UnaryOp(self, node):
         if isinstance(node.op, ast.USub):
             if isinstance(node.operand, (ast.Name, ast.Num)):
@@ -284,18 +301,28 @@ class ExpressionString(ast.NodeVisitor):
         return repr(node.s)
 
     def visit_BinOp(self, node):
-        if isinstance(node.op, ast.Mult):
-            return "({} * {})".format(self.visit(node.left), self.visit(node.right))
-        if isinstance(node.op, ast.Div):
-            return "({} / {})".format(self.visit(node.left), self.visit(node.right))
-        if isinstance(node.op, ast.Add):
-            return "({} + {})".format(self.visit(node.left), self.visit(node.right))
-        if isinstance(node.op, ast.Sub):
-            return "({} - {})".format(self.visit(node.left), self.visit(node.right))
-        if isinstance(node.op, ast.Pow):
-            return "({} ** {})".format(self.visit(node.left), self.visit(node.right))
-        else:
-            return "error"
+        newline = indent = ""
+        if self.pretty:
+            indent = "  " * self.indent
+            newline = "\n"
+        self.indent += 1
+        left = "{}{}{}".format(newline, indent, self.visit(node.left))
+        right = "{}{}{}".format(newline, indent, self.visit(node.right))
+        try:
+            if isinstance(node.op, ast.Mult):
+                return "({left} * {right})".format(left=left, right=right)
+            if isinstance(node.op, ast.Div):
+                return "({left} / {right})".format(left=left, right=right)
+            if isinstance(node.op, ast.Add):
+                return "({left} + {right})".format(left=left, right=right)
+            if isinstance(node.op, ast.Sub):
+                return "({left} - {right})".format(left=left, right=right)
+            if isinstance(node.op, ast.Pow):
+                return "({left} ** {right})".format(left=left, right=right)
+            else:
+                return "error"
+        finally:
+            self.indent -= 1
 
     op_translate = {ast.Lt: "<", ast.LtE: "<=", ast.Gt: ">", ast.GtE: ">=", ast.Eq: "==", ast.NotEq: "!="}
     def visit_Compare(self, node):
@@ -310,7 +337,12 @@ class ExpressionString(ast.NodeVisitor):
 
 
 class SimplifyExpression(ast.NodeTransformer):
-    def visit_Num(self, node):
+
+    def visit_UnaryOp(self, node):
+        node.operand = self.visit(node.operand)
+        if isinstance(node.op, ast.USub):
+            if isinstance(node.operand, ast.Num) and node.operand.n == 0:
+                node = node.operand
         return node
 
     def visit_BinOp(self, node):
@@ -392,8 +424,8 @@ def parse_expression(expression_string):
     return expr.value
 
 
-def node_to_string(node):
-    return ExpressionString().visit(node)
+def node_to_string(node, pretty=False):
+    return ExpressionString(pretty=pretty).visit(node)
 
 
 def validate_func(name, function_set):
